/*
** Module   : ImageHelper.java
** Abstract : image helper methods common for different widgets
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20150122 Created initial version.
** 002 EVL 20150415 Adding methods to work with prepackaged button imeges btn-*-arrow.
*/ 

package com.goldencode.p2j.ui.client;

import java.io.*;
import java.util.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;

/**
 * Image helper methods to handle prending image loading.
 */
public class ImageHelper
{
   /** Map to store pending images for image widget. */
   private static Map<Integer, Object> imgMap     = new HashMap<Integer, Object>();
   
   /** Map to store pending images for button widget up image. */
   private static Map<Integer, Object> imgUpMap   = new HashMap<Integer, Object>();
   
   /** Map to store pending images for button widget down image. */
   private static Map<Integer, Object> imgDownMap = new HashMap<Integer, Object>();
   
   /** Map to store pending images for button widget disabled image. */
   private static Map<Integer, Object> imgInsMap  = new HashMap<Integer, Object>();
   
   /**
    * Saves the new image name for further loading at widget realization stage.
    * 
    * @param    wId
    *           ID of the widget will own the image.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive for buttons or image.
    * @param    fileName
    *           The file name for the image to be loaded.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public static boolean storePendingImageName(int wId, int type, String fileName)
   {
      if (fileName == null || fileName.length() == 0)
      {
         return false;
      }
      
      switch (type)
      {
         case ButtonConfig.IMAGE_IMG:
         {
            imgMap.put(wId, fileName);
            break;
         }
         case ButtonConfig.IMAGE_UP:
         {
            imgUpMap.put(wId, fileName);
            break;
         }
         case ButtonConfig.IMAGE_DOWN:
         {
            imgDownMap.put(wId, fileName);
            break;
         }
         case ButtonConfig.IMAGE_INS:
         {
            imgInsMap.put(wId, fileName);
            break;
         }
      }
      
      return true;
   }

   /**
    * Saves the new image data byte array for further loading at widget realization stage.
    * 
    * @param    wId
    *           ID of the widget will own the image.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive for buttons or image.
    * @param    imageBytes
    *           The byte array data for the image to be loaded.
    *
    * @return   <code>TRUE</code> if operation successful <code>FALSE</code> otherwise.
    */
   public static boolean storePendingImageData(int wId, int type, byte[] imageBytes)
   {
      if (imageBytes == null)
      {
         return false;
      }
      
      switch (type)
      {
         case ButtonConfig.IMAGE_IMG:
         {
            imgMap.put(wId, imageBytes);
            break;
         }
         case ButtonConfig.IMAGE_UP:
         {
            imgUpMap.put(wId, imageBytes);
            break;
         }
         case ButtonConfig.IMAGE_DOWN:
         {
            imgDownMap.put(wId, imageBytes);
            break;
         }
         case ButtonConfig.IMAGE_INS:
         {
            imgInsMap.put(wId, imageBytes);
            break;
         }
      }
      
      return true;
   }
   
   /**
    * Gets the new image data byte array for loading at widget realization stage.
    * 
    * @param    wId
    *           ID of the widget will own the image.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive for buttons or image.
    *
    * @return   The byte array data for the image to be loaded or null if there is no data.
    */
   public static byte[] getPendingImageData(int wId, int type)
   {
      byte[] imgData = null;
      Object imgObj = null;
      
      switch (type)
      {
         case ButtonConfig.IMAGE_IMG:
         {
            imgObj = imgMap.get(wId);
            // check if we have data array, not string
            if (imgObj != null && !(imgObj instanceof String))
            {
               imgData = (byte[])imgMap.remove(wId);
            }
            break;
         }
         case ButtonConfig.IMAGE_UP:
         {
            imgObj = imgUpMap.get(wId);
            // check if we have data array, not string
            if (imgObj != null && !(imgObj instanceof String))
            {
               imgData = (byte[])imgUpMap.remove(wId);
            }
            break;
         }
         case ButtonConfig.IMAGE_DOWN:
         {
            imgObj = imgDownMap.get(wId);
            // check if we have data array, not string
            if (imgObj != null && !(imgObj instanceof String))
            {
               imgData = (byte[])imgDownMap.remove(wId);
            }
            break;
         }
         case ButtonConfig.IMAGE_INS:
         {
            imgObj = imgInsMap.get(wId);
            // check if we have data array, not string
            if (imgObj != null && !(imgObj instanceof String))
            {
               imgData = (byte[])imgInsMap.remove(wId);
            }
            break;
         }
      }
      
      return imgData;
   }
   
   /**
    * Gets the new image file name for loading at widget realization stage.
    * 
    * @param    wId
    *           ID of the widget will own the image.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive for buttons or image.
    *
    * @return   The file name for the image to be loaded and found in PROPATH directory.
    */
   public static String getPendingImageName(int wId, int type)
   {
      String imgName = null;
      Object imgObj = null;
      
      switch (type)
      {
         case ButtonConfig.IMAGE_IMG:
         {
            imgObj = imgMap.get(wId);
            // check if we have file name string
            if (imgObj != null && imgObj instanceof String)
            {
               imgName = (String)imgMap.remove(wId);
            }
            break;
         }
         case ButtonConfig.IMAGE_UP:
         {
            imgObj = imgUpMap.get(wId);
            // check if we have file name string
            if (imgObj != null && imgObj instanceof String)
            {
               imgName = (String)imgUpMap.remove(wId);
            }
            break;
         }
         case ButtonConfig.IMAGE_DOWN:
         {
            imgObj = imgDownMap.get(wId);
            // check if we have file name string
            if (imgObj != null && imgObj instanceof String)
            {
               imgName = (String)imgDownMap.remove(wId);
            }
            break;
         }
         case ButtonConfig.IMAGE_INS:
         {
            imgObj = imgInsMap.get(wId);
            // check if we have file name string
            if (imgObj != null && imgObj instanceof String)
            {
               imgName = (String)imgInsMap.remove(wId);
            }
            break;
         }
      }
      
      return ThinClient.getLocalImageName(imgName);
   }

   /**
    * Checks if the given image file name belongs to the set of the prepackaged images embedded
    * into project distribution.  Currently the embedded images includes: btn-up-arrow,
    * btn-down-arrow, btn-left-arrow and btn-right-arrow.
    * 
    * @param    fileName
    *           The file name for the image to be checked.
    *
    * @return   <code>TRUE</code> if given image is prepackaged <code>FALSE</code> otherwise.
    */
   public static boolean isPrepackaged(String fileName)
   {
      // simple check for input parameters
      if (fileName == null || fileName.isEmpty())
      {
         return false;
      }
      
      // make case insensitive
      String nameLCase = fileName.toLowerCase();
      
      return nameLCase.indexOf("btn-up-arrow") != -1 ||
             nameLCase.indexOf("btn-down-arrow") != -1 ||
             nameLCase.indexOf("btn-left-arrow") != -1 ||
             nameLCase.indexOf("btn-right-arrow") != -1;
   }
   
   /**
    * Gets the new image data byte array for prepackaged images.
    * 
    * @param    fileName
    *           The file name for the image to be loaded from p2j.jar.
    *
    * @return   The byte array data for the image to be loaded or null if there is no data.
    */
   public static byte[] getPrepackagedImageData(String fileName)
   {
      byte[] imgData = null;
      
      // loading image embedded into p2j.jar
      InputStream is = ImageHelper.class.getResourceAsStream(fileName.toLowerCase());
         
      if (is != null)
      {
         imgData = UiUtils.getData(is);
      }
         
      return imgData;
   }
}

